@@ -40,7 +40,7 @@
 
    -----
 
-   canon_pp.c: $Revision: 1.14 $
+   canon_pp.c: $Revision$
 
    This file is part of the canon_pp backend, supporting Canon FBX30P 
    and NX40P scanners
@@ -61,7 +61,7 @@
 #endif
 
 #ifndef VERSION
-#define VERSION "$Revision: 1.14 $"
+#define VERSION "$Revision$"
 #endif
 
 #include  <string.h>
@@ -114,7 +114,11 @@ static SANE_Bool force_nibble = SANE_FALSE;
 /* Constants */
 
 /* Colour Modes */
-static const SANE_String_Const cmodes[] = { "Greyscale", "Colour", NULL };
+static const SANE_String_Const cmodes[] = { 
+	SANE_VALUE_SCAN_MODE_GRAY, 
+	SANE_VALUE_SCAN_MODE_COLOR, 
+	NULL };
+
 /* bit depths */
 static const SANE_String_Const depths[] = { "8", "12", NULL };
 /* resolutions */
@@ -151,7 +155,7 @@ sane_init (SANE_Int *vc, SANE_Auth_Callback cb)
 #endif
 
 	if(vc)
-		*vc = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+		*vc = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
 
 	DBG(2,"sane_init: >> ieee1284_find_ports\n");
 	/* Find lp ports */
