@@ -44,7 +44,7 @@
 
 /* Please increase version number with every change 
    (don't forget to update dll.desc) */
-#define DLL_VERSION "1.0.11"
+#define DLL_VERSION "1.0.12"
 
 #ifdef _AIX
 # include "lalloca.h"		/* MUST come first for AIX! */
@@ -57,7 +57,7 @@
 #include <posix/dirent.h>
 #endif
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 #include "lalloca.h"
 
 #include <errno.h>
@@ -95,12 +95,14 @@
 #endif
 
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <dirent.h>
 
-#include "sane/sane.h"
-#include "sane/sanei.h"
+#include "../include/sane/sane.h"
+#include "../include/sane/sanei.h"
 
 #define BACKEND_NAME dll
-#include "sane/sanei_backend.h"
+#include "../include/sane/sanei_backend.h"
 
 #ifndef PATH_MAX
 # define PATH_MAX       1024
@@ -108,6 +110,8 @@
 
 #if defined(HAVE_OS2_H)
 # define DIR_SEP        ";"
+#elif defined(__CYGWIN__)
+# define DIR_SEP        ":"
 #elif defined(HAVE_WINDOWS_H)
 # define DIR_SEP        ";"
 #else
@@ -115,7 +119,7 @@
 #endif
 
 
-#include "sane/sanei_config.h"
+#include "../include/sane/sanei_config.h"
 #define DLL_CONFIG_FILE "dll.conf"
 #define DLL_ALIASES_FILE "dll.aliases"
 
@@ -162,7 +166,7 @@ struct backend
   u_int loaded:1;		/* are the functions available? */
   u_int inited:1;		/* has the backend been initialized? */
   void *handle;			/* handle returned by dlopen() */
-  void *(*op[NUM_OPS]) ();
+  void *(*op[NUM_OPS]) (void);
 };
 
 #define BE_ENTRY(be,func)       sane_##be##_##func
@@ -207,7 +211,7 @@ struct backend
 }
 
 #ifndef __BEOS__
-#include "dll-preload.c"
+#include "dll-preload.h"
 #endif
 
 struct meta_scanner
@@ -322,13 +326,13 @@ dyld_get_error_str ()
 static SANE_Status
 load (struct backend *be)
 {
-	// use BeOS kernel function to load scanner addons from ~/config/add-ons/SANE/
+	/* use BeOS kernel function to load scanner addons from ~/config/add-ons/SANE */
 	char path[PATH_MAX];
 	image_id id = -1;
 	int i, w;
 	directory_which which[3] = { B_USER_ADDONS_DIRECTORY, B_COMMON_ADDONS_DIRECTORY, B_BEOS_ADDONS_DIRECTORY };
 	
-	// look for config files in SANE/conf
+	/* look for config files in SANE/conf */
 	for (w = 0; (w < 3) && (id < 0) && (find_directory(which[w],0,true,path,PATH_MAX) == 0); w++)
 	{
 		strcat(path,"/SANE/");
@@ -382,14 +386,17 @@ load (struct backend *be)
 # define PREFIX "libsane-"
 # ifdef __hpux
 #   define POSTFIX ".sl.%u"
-#elif defined (HAVE_WINDOWS_H)
+#   define ALT_POSTFIX ".so.%u"
+# elif defined (HAVE_WINDOWS_H)
 #   undef PREFIX
 #   define PREFIX "cygsane-"
 #   define POSTFIX "-%u.dll"
-#elif defined (HAVE_OS2_H)
+# elif defined (HAVE_OS2_H)
 #   undef PREFIX
 #   define PREFIX ""
 #   define POSTFIX ".dll"
+# elif defined (__APPLE__) && defined (__MACH__)
+#   define POSTFIX ".%u.so"
 # else
 #   define POSTFIX ".so.%u"
 # endif
@@ -461,6 +468,18 @@ load (struct backend *be)
 	break;
       DBG (4, "load: couldn't open `%s' (%s)\n", libname, strerror (errno));
 
+#ifdef ALT_POSTFIX      
+      /* Some platforms have two ways of storing their libraries, try both
+	 postfixes */
+      snprintf (libname, sizeof (libname), "%s/" PREFIX "%s" ALT_POSTFIX,
+		dir, be->name, V_MAJOR);
+      DBG (4, "load: trying to load `%s'\n", libname);
+      fp = fopen (libname, "r");
+      if (fp)
+	break;
+      DBG (4, "load: couldn't open `%s' (%s)\n", libname, strerror (errno));
+#endif
+
       dir = strsep (&src, DIR_SEP);
     }
   if (orig_src)
@@ -595,11 +614,11 @@ init (struct backend *be)
   if (status != SANE_STATUS_GOOD)
     return status;
 
-  if (SANE_VERSION_MAJOR (version) != V_MAJOR)
+  if (SANE_VERSION_MAJOR (version) != SANE_CURRENT_MAJOR)
     {
       DBG (1,
 	   "init: backend `%s' has a wrong major version (%d instead of %d)\n",
-	   be->name, SANE_VERSION_MAJOR (version), V_MAJOR);
+	   be->name, SANE_VERSION_MAJOR (version), SANE_CURRENT_MAJOR);
       return SANE_STATUS_INVAL;
     }
   DBG (4, "init: backend `%s' is version %d.%d.%d\n", be->name,
@@ -703,12 +722,113 @@ add_alias (const char *line_param)
 }
 
 
+static void
+read_config (const char *conffile)
+{
+  FILE *fp;
+  char config_line[PATH_MAX];
+  char *backend_name;
+
+  fp = sanei_config_open (conffile);
+  if (!fp)
+    {
+      DBG (1, "sane_init/read_config: Couldn't open config file (%s): %s\n",
+           conffile, strerror (errno));
+      return; /* don't insist on config file */
+    }
+
+  DBG (5, "sane_init/read_config: reading %s\n", conffile);
+  while (sanei_config_read (config_line, sizeof (config_line), fp))
+    {
+      char *comment;
+      SANE_String_Const cp;
+
+      cp = sanei_config_get_string (config_line, &backend_name);
+      /* ignore empty lines */
+      if (!backend_name || cp == config_line)
+        {
+          if (backend_name)
+            free (backend_name);
+          continue;
+        }
+      /* ignore line comments */
+      if (backend_name[0] == '#')
+        {
+          free (backend_name);
+          continue;
+        }
+      /* ignore comments after backend names */
+      comment = strchr (backend_name, '#');
+      if (comment)
+        *comment = '\0';
+      add_backend (backend_name, 0);
+      free (backend_name);
+    }
+  fclose (fp);
+}
+
+static void
+read_dlld (void)
+{
+  DIR *dlld;
+  struct dirent *dllconf;
+  struct stat st;
+  char conffile[PATH_MAX];
+  size_t len, plen;
+
+  DBG (5, "sane_init/read_dlld: processing %s ...\n",
+       STRINGIFY(PATH_SANE_CONFIG_DIR) "/dll.d");
+
+  /* Debian specific: read files under $sysconfdir/sane.d/dll.d */
+  dlld = opendir (STRINGIFY(PATH_SANE_CONFIG_DIR) "/dll.d");
+
+  if (dlld == NULL)
+    {
+      DBG (1, "sane_init/read_dlld: opendir failed: %s\n",
+           strerror (errno));
+      return;
+    }
+
+  plen = strlen (STRINGIFY(PATH_SANE_CONFIG_DIR)) + 1;
+
+  while ((dllconf = readdir (dlld)) != NULL)
+    {
+      /* dotfile (or directory) */
+      if (dllconf->d_name[0] == '.')
+        continue;
+
+      len = strlen (dllconf->d_name);
+
+      /* backup files */
+      if ((dllconf->d_name[len-1] == '~')
+          || (dllconf->d_name[len-1] == '#'))
+        continue;
+
+      snprintf (conffile, PATH_MAX, "%s/dll.d/%s",
+                STRINGIFY (PATH_SANE_CONFIG_DIR), dllconf->d_name);
+
+      DBG (5, "sane_init/read_dlld: considering %s\n", conffile);
+
+      if (lstat (conffile, &st) != 0)
+        continue;
+
+      if (!S_ISREG (st.st_mode))
+        continue;
+
+      /* expects a path relative to PATH_SANE_CONFIG_DIR */
+      read_config (conffile+plen);
+    }
+
+  closedir (dlld);
+
+  DBG (5, "sane_init/read_dlld: done.\n");
+}
+
 SANE_Status
 sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 {
 #ifndef __BEOS__
   char config_line[PATH_MAX];
-  char *backend_name;
   size_t len;
   FILE *fp;
   int i;
@@ -745,42 +865,12 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
     *version_code = SANE_VERSION_CODE (SANE_DLL_V_MAJOR, SANE_DLL_V_MINOR,
 				       SANE_DLL_V_BUILD);
 
-  fp = sanei_config_open (DLL_CONFIG_FILE);
-  if (!fp)
-    {
-      DBG (1, "sane_init: Couldn't open config file (%s): %s\n",
-	   DLL_CONFIG_FILE, strerror (errno));
-      return SANE_STATUS_GOOD;	/* don't insist on config file */
-    }
-
-  DBG (5, "sane_init: reading %s\n", DLL_CONFIG_FILE);
-  while (sanei_config_read (config_line, sizeof (config_line), fp))
-    {
-      char *comment;
-      SANE_String_Const cp;
-
-      cp = sanei_config_get_string (config_line, &backend_name);
-      /* ignore empty lines */
-      if (!backend_name || cp == config_line)
-	{
-	  if (backend_name)
-	    free (backend_name);
-	  continue;
-	}
-      /* ignore line comments */
-      if (backend_name[0] == '#')
-	{
-	  free (backend_name);
-	  continue;
-	}
-      /* ignore comments after backend names */
-      comment = strchr (backend_name, '#');
-      if (comment)
-	*comment = '\0';
-      add_backend (backend_name, 0);
-      free (backend_name);
-    }
-  fclose (fp);
+  /*
+   * Read dll.conf & dll.d
+   * Read dll.d first, so that the extras backends will be tried last
+   */
+  read_dlld ();
+  read_config (DLL_CONFIG_FILE);
 
   fp = sanei_config_open (DLL_ALIASES_FILE);
   if (!fp)
@@ -801,8 +891,8 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   fclose (fp);
 
 #else  
-	// no ugly config files, just get scanners from their ~/config/add-ons/SANE
-	// look for drivers
+	/* no ugly config files, just get scanners from their ~/config/add-ons/SANE */
+	/* look for drivers */
 	for (i = 0; i < 3; i++)
 	{
 		if (find_directory(which[i],0,true,path,1024) < B_OK)
@@ -844,8 +934,8 @@ sane_exit (void)
 	      (*(op_exit_t)be->op[OP_EXIT]) ();
 	    }
 #ifdef __BEOS__
-			// use BeOS kernel functions to unload add-ons
-			if(be->handle) unload_add_on((image_id)be->handle);	
+	  /* use BeOS kernel functions to unload add-ons */
+	  if(be->handle) unload_add_on((image_id)be->handle);
 #else
 #ifdef HAVE_DLL
 
