@@ -3,8 +3,8 @@
    Copyright (C) 2003 Oliver Rauch
    Copyright (C) 2003, 2004 Henning Meier-Geinitz <henning@meier-geinitz.de>
    Copyright (C) 2004, 2005 Gerhard Jaeger <gerhard@gjaeger.de>
-   Copyright (C) 2004, 2005 Stephane Voltz <stef.dev@free.fr>
-   Copyright (C) 2005 Pierre Willenbrock <pierre@pirsoft.dnsalias.org>
+   Copyright (C) 2004-2010 Stphane Voltz <stef.dev@free.fr>
+   Copyright (C) 2005-2009 Pierre Willenbrock <pierre@pirsoft.dnsalias.org>
    Copyright (C) 2006 Laurent Charpentier <laurent_pubs@yahoo.com>
    Parts of the structs have been taken from the gt68xx backend by
    Sergey Vlasov <vsu@altlinux.ru> et al.
@@ -52,6 +52,9 @@
 #define GENESYS_LOW_H
 
 #include <stddef.h>
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
 #include "../include/sane/sane.h"
 
 #define DBG_error0      0	/* errors/warnings printed even with devuglevel 0 */
@@ -69,14 +72,12 @@
     if (status != SANE_STATUS_GOOD) return status;      \
   } while (SANE_FALSE)
 
-#define MM_PER_INCH 25.4
 
 /* Flags */
 #define GENESYS_FLAG_UNTESTED     (1 << 0)	/* Print a warning for these scanners */
 #define GENESYS_FLAG_14BIT_GAMMA  (1 << 1)	/* use 14bit Gamma table instead of 12 */
 #define GENESYS_FLAG_LAZY_INIT    (1 << 2)	/* skip extensive ASIC test at init   */
-#define GENESYS_FLAG_USE_PARK     (1 << 3)	/* use genesys_park_head() instead of
-						   genesys_slow_back_home()           */
+#define GENESYS_FLAG_XPA          (1 << 3)
 #define GENESYS_FLAG_SKIP_WARMUP  (1 << 4)	/* skip genesys_warmup()              */
 #define GENESYS_FLAG_OFFSET_CALIBRATION   (1 << 5)	/* do offset calibration      */
 #define GENESYS_FLAG_SEARCH_START (1 << 6)	/* do start search beofre scanning    */
@@ -94,7 +95,20 @@
 #define GENESYS_FLAG_ALT_SLOPE_CREATE (1 << 11)	/* use alternative slope
 						   creation function */
 
-#define GENESYS_FLAG_DARK_WHITE_CALIBRATION (1 << 12) /*yet another calibration method. does white and dark shading in one run, depending on a black and a white strip*/
+#define GENESYS_FLAG_DARK_WHITE_CALIBRATION (1 << 12) /* yet another calibration method. does white and dark shading in one run, depending on a black and a white strip*/
+#define GENESYS_FLAG_CUSTOM_GAMMA     (1 << 13)       /* allow custom gamma tables */
+#define GENESYS_FLAG_NO_CALIBRATION   (1 << 14)       /* allow scanners to use skip the calibration, needed for sheetfed scanners */
+#define GENESYS_FLAG_HALF_CCD_MODE    (1 << 15)       /* scanner has setting for half ccd mode */
+
+#define GENESYS_HAS_NO_BUTTONS       0              /* scanner has no supported button */
+#define GENESYS_HAS_SCAN_SW          (1 << 0)       /* scanner has SCAN button */
+#define GENESYS_HAS_FILE_SW          (1 << 1)       /* scanner has FILE button */
+#define GENESYS_HAS_COPY_SW          (1 << 2)       /* scanner has COPY button */
+#define GENESYS_HAS_EMAIL_SW         (1 << 3)       /* scanner has EMAIL button */
+#define GENESYS_HAS_PAGE_LOADED_SW   (1 << 4)       /* scanner has paper in detection */
+#define GENESYS_HAS_OCR_SW           (1 << 5)       /* scanner has OCR button */
+#define GENESYS_HAS_POWER_SW         (1 << 6)       /* scanner has power button */
+#define GENESYS_HAS_CALIBRATE        (1 << 7)       /* scanner has 'calibrate' software button to start calibration */
 
 /* USB control message values */
 #define REQUEST_TYPE_IN		(USB_TYPE_VENDOR | USB_DIR_IN)
@@ -123,6 +137,8 @@
 #define BULK_REGISTER		0x11
 
 #define BULKIN_MAXSIZE          0xFE00
+#define GL646_BULKIN_MAXSIZE    0xFFC0
+#define GL646_BULKIN_MINSIZE    0x0800
 #define BULKOUT_MAXSIZE         0xF000
 
 /* AFE values */
@@ -130,10 +146,10 @@
 #define AFE_SET        2
 #define AFE_POWER_SAVE 4
 
-#define LOWORD(x)  ((u_int16_t)(x & 0xffff))
-#define HIWORD(x)  ((u_int16_t)(x >> 16))
-#define LOBYTE(x)  ((u_int8_t)((x) & 0xFF))
-#define HIBYTE(x)  ((u_int8_t)((x) >> 8))
+#define LOWORD(x)  ((uint16_t)(x & 0xffff))
+#define HIWORD(x)  ((uint16_t)(x >> 16))
+#define LOBYTE(x)  ((uint8_t)((x) & 0xFF))
+#define HIBYTE(x)  ((uint8_t)((x) >> 8))
 
 /* Global constants */
 /* todo: check if those are the same for every scanner */
@@ -152,15 +168,17 @@ typedef struct
 
 typedef struct
 {
-  u_int8_t reg[4];
-  u_int8_t sign[3];
-  u_int8_t offset[3];
-  u_int8_t gain[3];
-  u_int8_t reg2[3];
+  uint8_t fe_id;	      /**> id of the frontend description */
+  uint8_t reg[4];
+  uint8_t sign[3];
+  uint8_t offset[3];
+  uint8_t gain[3];
+  uint8_t reg2[3];
 } Genesys_Frontend;
 
 typedef struct
 {
+  uint8_t sensor_id;	      /**> id of the sensor description */
   int optical_res;
   int black_pixels;
   int dummy_pixel;              /* value of dummy register. */
@@ -168,21 +186,22 @@ typedef struct
   int sensor_pixels;		/* total pixels used by the sensor */
   int fau_gain_white_ref;	/* TA CCD target code (reference gain) */
   int gain_white_ref;		/* CCD target code (reference gain) */
-  u_int8_t regs_0x08_0x0b[4];
-  u_int8_t regs_0x10_0x1d[14];
-  u_int8_t regs_0x52_0x5e[13];
+  uint8_t regs_0x08_0x0b[4];
+  uint8_t regs_0x10_0x1d[14];
+  uint8_t regs_0x52_0x5e[13];
   float red_gamma;
   float green_gamma;
   float blue_gamma;
-  u_int16_t *red_gamma_table;
-  u_int16_t *green_gamma_table;
-  u_int16_t *blue_gamma_table;
+  uint16_t *red_gamma_table;
+  uint16_t *green_gamma_table;
+  uint16_t *blue_gamma_table;
 } Genesys_Sensor;
 
 typedef struct
 {
-  u_int8_t value[2];
-  u_int8_t enable[2];
+  uint8_t gpo_id;	/**> id of the gpo description */
+  uint8_t value[2];
+  uint8_t enable[2];
 } Genesys_Gpo;
 
 typedef struct
@@ -199,6 +218,7 @@ typedef struct
 
 typedef struct
 {
+  uint8_t motor_id;	         /**> id of the motor description */
   SANE_Int base_ydpi;		 /* motor base steps. Unit: 1/" */
   SANE_Int optical_ydpi;	 /* maximum resolution in y-direction. Unit: 1/"  */
   SANE_Int max_step_type;        /* maximum step type. 0-2 */
@@ -224,13 +244,17 @@ Genesys_Color_Order;
 /*135 registers for gl841 + 1 null-reg*/
 #define GENESYS_MAX_REGS 136
 
-#define DAC_WOLFSON_UMAX 0
-#define DAC_WOLFSON_ST12 1
-#define DAC_WOLFSON_ST24 2
-#define DAC_WOLFSON_5345 3
+#define DAC_WOLFSON_UMAX   0
+#define DAC_WOLFSON_ST12   1
+#define DAC_WOLFSON_ST24   2
+#define DAC_WOLFSON_5345   3
 #define DAC_WOLFSON_HP2400 4
 #define DAC_WOLFSON_HP2300 5
-#define DAC_CANONLIDE35  6
+#define DAC_CANONLIDE35    6
+#define DAC_AD_XP200       7   /* Analog Device frontend */
+#define DAC_WOLFSON_XP300  8
+#define DAC_WOLFSON_HP3670 9
+#define DAC_WOLFSON_DSM600 10
 
 #define CCD_UMAX         0
 #define CCD_ST12         1	/* SONY ILX548: 5340 Pixel  ??? */
@@ -239,6 +263,14 @@ Genesys_Color_Order;
 #define CCD_HP2400       4
 #define CCD_HP2300       5
 #define CCD_CANONLIDE35  6
+#define CIS_XP200        7      /* CIS sensor for Strobe XP200 */
+				/* 8 is unused currently */
+#define CCD_HP3670       9
+#define CCD_DP665        10
+#define CCD_ROADWARRIOR  11
+#define CCD_DSMOBILE600  12
+#define CCD_XP300        13
+#define CCD_DP685        14
 
 #define GPO_UMAX         0
 #define GPO_ST12         1
@@ -247,6 +279,11 @@ Genesys_Color_Order;
 #define GPO_HP2400       4
 #define GPO_HP2300       5
 #define GPO_CANONLIDE35  6
+#define GPO_XP200        7
+#define GPO_XP300        8
+#define GPO_HP3670       9
+#define GPO_DP665        10
+#define GPO_DP685        11
 
 #define MOTOR_UMAX       0
 #define MOTOR_5345       1
@@ -254,10 +291,18 @@ Genesys_Color_Order;
 #define MOTOR_HP2400     3
 #define MOTOR_HP2300     4
 #define MOTOR_CANONLIDE35 5
+#define MOTOR_XP200      6
+#define MOTOR_XP300      7
+#define MOTOR_HP3670     9
+#define MOTOR_DP665      10
+#define MOTOR_ROADWARRIOR 11
+#define MOTOR_DSMOBILE_600 12
 
 
 /* Forward typedefs */
 typedef struct Genesys_Device Genesys_Device;
+struct Genesys_Scanner;
+typedef struct Genesys_Calibration_Cache  Genesys_Calibration_Cache;
 
 /**
  * Scanner command set description.
@@ -297,12 +342,14 @@ typedef struct Genesys_Command_Set
 
   int (*bulk_full_size) (void);
 
-    SANE_Status (*set_fe) (Genesys_Device * dev, u_int8_t set);
+    SANE_Status (*set_fe) (Genesys_Device * dev, uint8_t set);
     SANE_Status (*set_powersaving) (Genesys_Device * dev, int delay);
     SANE_Status (*save_power) (Genesys_Device * dev, SANE_Bool enable);
 
   void (*set_motor_power) (Genesys_Register_Set * regs, SANE_Bool set);
-  void (*set_lamp_power) (Genesys_Register_Set * regs, SANE_Bool set);
+  void (*set_lamp_power) (Genesys_Device * dev, 
+			  Genesys_Register_Set * regs, 
+			  SANE_Bool set);
 
     SANE_Status (*begin_scan) (Genesys_Device * dev,
 			       Genesys_Register_Set * regs,
@@ -320,18 +367,55 @@ typedef struct Genesys_Command_Set
 
     SANE_Status (*slow_back_home) (Genesys_Device * dev,
 				   SANE_Bool wait_until_home);
-    SANE_Status (*park_head) (Genesys_Device * dev,
-			      Genesys_Register_Set * reg,
-			      SANE_Bool wait_until_home);
+
     SANE_Status (*bulk_write_register) (Genesys_Device * dev,
 					Genesys_Register_Set * reg, 
 					size_t elems);
-    SANE_Status (*bulk_write_data) (Genesys_Device * dev, u_int8_t addr, 
-				    u_int8_t * data, size_t len);
+    SANE_Status (*bulk_write_data) (Genesys_Device * dev, uint8_t addr, 
+				    uint8_t * data, size_t len);
 
-    SANE_Status (*bulk_read_data) (Genesys_Device * dev, u_int8_t addr,
-				   u_int8_t * data, size_t len);
+    SANE_Status (*bulk_read_data) (Genesys_Device * dev, uint8_t addr,
+				   uint8_t * data, size_t len);
 
+  /* Updates hardware sensor information in Genesys_Scanner.val[].
+     If possible, just get information for given option.
+     The sensor state in Genesys_Scanner.val[] should be merged with the
+     new sensor state, using the information that was last read by the frontend
+     in Genesys_Scanner.last_val[], in such a way that a button up/down 
+     relative to Genesys_Scanner.last_val[] is not lost.
+   */
+  SANE_Status (*update_hardware_sensors) (struct Genesys_Scanner * s);
+
+    /* functions for sheetfed scanners */
+    /**
+     * load document into scanner
+     */
+    SANE_Status (*load_document) (Genesys_Device * dev);
+    /**
+     * detects is the scanned document has left scanner. In this
+     * case it updates the amount of data to read and set up
+     * flags in the dev struct
+     */
+    SANE_Status (*detect_document_end) (Genesys_Device * dev);
+    /**
+     * eject document from scanner
+     */
+    SANE_Status (*eject_document) (Genesys_Device * dev);
+    /**
+     * search for an black or white area in forward or reverse 
+     * direction */
+    SANE_Status (*search_strip) (Genesys_Device * dev, SANE_Bool forward, SANE_Bool black);
+
+    SANE_Status (*is_compatible_calibration) (
+	Genesys_Device * dev,
+	Genesys_Calibration_Cache *cache,
+        SANE_Bool for_overwrite);
+
+    /* functions for transparency adapter */
+    /**
+     * move scanning head to transparency adapter
+     */
+    SANE_Status (*move_to_ta) (Genesys_Device * dev);
 } Genesys_Command_Set;
 
 typedef struct Genesys_Model
@@ -349,7 +433,8 @@ typedef struct Genesys_Model
   SANE_Int bpp_color_values[MAX_DPI];	/* possible depths in color mode */
 
   SANE_Fixed x_offset;		/* Start of scan area in mm */
-  SANE_Fixed y_offset;		/* Start of scan area in mm */
+  SANE_Fixed y_offset;		/* Start of scan area in mm (Amount of 
+				   feeding needed to get to the medium) */
   SANE_Fixed x_size;		/* Size of scan area in mm */
   SANE_Fixed y_size;		/* Size of scan area in mm */
 
@@ -363,6 +448,11 @@ typedef struct Genesys_Model
 
   SANE_Fixed y_offset_calib_ta;	/* Start of white strip in TA mode in mm */
 
+  SANE_Fixed post_scan;		/* Size of scan area after paper sensor stops
+				   sensing document in mm */
+  SANE_Fixed eject_feed;	/* Amount of feeding needed to eject document 
+				   after finishing scanning in mm */
+
   /* Line-distance correction (in pixel at optical_ydpi) for CCD scanners */
   SANE_Int ld_shift_r;		/* red */
   SANE_Int ld_shift_g;		/* green */
@@ -371,43 +461,63 @@ typedef struct Genesys_Model
   Genesys_Color_Order line_mode_color_order;	/* Order of the CCD/CIS colors */
 
   SANE_Bool is_cis;		/* Is this a CIS or CCD scanner? */
+  SANE_Bool is_sheetfed;	/* Is this sheetfed scanner? */
 
   SANE_Int ccd_type;		/* which SENSOR type do we have ? */
   SANE_Int dac_type;		/* which DAC do we have ? */
   SANE_Int gpo_type;		/* General purpose output type */
   SANE_Int motor_type;		/* stepper motor type */
   SANE_Word flags;		/* Which hacks are needed for this scanner? */
+  SANE_Word buttons;		/* Button flags, described existing buttons for the model */
   /*@} */
   SANE_Int shading_lines;	/* how many lines are used for shading calibration */
   SANE_Int search_lines;	/* how many lines are used to search start position */
 } Genesys_Model;
 
+#define SCAN_METHOD_FLATBED      0     /**> normal scan method */
+#define SCAN_METHOD_TRANSPARENCY 2     /**> scan using transparency adaptor */
+#define SCAN_METHOD_NEGATIVE     0x88  /**> scan using negative adaptor */
+
+#define SCAN_MODE_LINEART        0 	/**> lineart scan mode */
+#define SCAN_MODE_HALFTONE       1 	/**> halftone scan mode */
+#define SCAN_MODE_GRAY           2 	/**> gray scan mode */
+#define SCAN_MODE_COLOR          4 	/**> color scan mode */
 
 typedef struct
 {
   int scan_method;		/* todo: change >=2: Transparency, 0x88: negative film */
   int scan_mode;		/* todo: change 0,1 = lineart, halftone; 2 = gray, 3 = 3pass color, 4=single pass color */
-  int xres;			/* dpi */
-  int yres;			/* dpi */
+  int xres;			/**> horizontal dpi */
+  int yres;			/**> vertical dpi */
 
   double tl_x;			/* x start on scan table in mm */
   double tl_y;			/* y start on scan table in mm */
 
-  unsigned int lines;		/* number of lines at scan resolution */
-  unsigned int pixels;		/* number of pixels at scan resolution */
+  unsigned int lines;		/**> number of lines at scan resolution */
+  unsigned int pixels;		/**> number of pixels at scan resolution */
 
   unsigned int depth;/* bit depth of the scan */
 
   /* todo : remove these fields ? */
   int exposure_time;
 
-  unsigned int color_filter;	/* todo: check, may be make it an advanced option */
+  unsigned int color_filter;
 
-  /* BW threshold */
+  /**> true if scan is true gray, false if monochrome scan */
+  int true_gray;	
+
+  /**> lineart threshold */
   int threshold;
 
-  /* Disable interpolation for xres<yres*/
+  /**> lineart threshold curve for dynamic rasterization */
+  int threshold_curve;
+
+  /**> Disable interpolation for xres<yres*/
   int disable_interpolation;
+
+  /**> true is lineart is generated from gray data by
+   * the dynamic rasterization algo */
+  int dynamic_lineart;
 } Genesys_Settings;
 
 typedef struct Genesys_Current_Setup
@@ -416,6 +526,7 @@ typedef struct Genesys_Current_Setup
     int lines;          /* line count expected from scanner */
     int depth;          /* depth expected from scanner */
     int channels;       /* channel count expected from scanner */
+    int scan_method;	/* scanning method: flatbed or XPA */
     int exposure_time;  /* used exposure time */
     float xres;         /* used xres */
     float yres;         /* used yres*/
@@ -432,10 +543,28 @@ typedef struct Genesys_Buffer
   size_t avail;	/* data bytes currently in buffer */
 } Genesys_Buffer;
 
+struct Genesys_Calibration_Cache
+{
+  Genesys_Current_Setup used_setup;/* used to check if entry is compatible */
+  time_t last_calibration;
+
+  Genesys_Frontend frontend;
+  Genesys_Sensor sensor;
+
+  size_t calib_pixels;
+  size_t calib_channels;
+  size_t average_size;
+  uint8_t *white_average_data;
+  uint8_t *dark_average_data;
+
+  struct Genesys_Calibration_Cache *next;
+};
+
 struct Genesys_Device
 {
   SANE_Int dn;
   SANE_String file_name;
+  SANE_String calib_file;
   Genesys_Model *model;
 
   Genesys_Register_Set reg[GENESYS_MAX_REGS];
@@ -445,19 +574,25 @@ struct Genesys_Device
   Genesys_Sensor sensor;
   Genesys_Gpo gpo;
   Genesys_Motor motor;
-  u_int16_t slope_table0[256];
-  u_int16_t slope_table1[256];
+  uint16_t slope_table0[256];
+  uint16_t slope_table1[256];
+  uint8_t  control[6];
   time_t init_date;
 
-  u_int8_t *white_average_data;
-  u_int8_t *dark_average_data;
-  u_int16_t dark[3];
+  size_t average_size;
+  size_t calib_pixels;
+  size_t calib_channels;
+  uint8_t *white_average_data;
+  uint8_t *dark_average_data;
+  uint16_t dark[3];
 
   SANE_Bool already_initialized;
   SANE_Int scanhead_position_in_steps;
   SANE_Int lamp_off_time;
 
   SANE_Bool read_active;
+  SANE_Bool document;		/**> for sheetfed scanner's, is TRUE when there
+				   is a document in the scanner */
 
   Genesys_Buffer read_buffer;
   Genesys_Buffer lines_buffer;
@@ -468,8 +603,14 @@ struct Genesys_Device
 
   size_t total_bytes_read;	/* total bytes read sent to frontend */
   size_t total_bytes_to_read;	/* total bytes read to be sent to frontend */
+  size_t wpl;			/* asic's word per line */
 
   Genesys_Current_Setup current_setup; /* contains the real used values */
+ 
+  /**> look up table used in dynamic rasterization */
+  unsigned char lineart_lut[256];
+
+  Genesys_Calibration_Cache *calibration_cache;
 
   struct Genesys_Device *next;
 };
@@ -497,15 +638,15 @@ sanei_genesys_set_reg_from_set (Genesys_Register_Set * regs,
 				SANE_Byte address, SANE_Byte value);
 
 extern SANE_Status
-sanei_genesys_read_register (Genesys_Device * dev, u_int8_t reg,
-			     u_int8_t * val);
+sanei_genesys_read_register (Genesys_Device * dev, uint8_t reg,
+			     uint8_t * val);
 
 extern SANE_Status
-sanei_genesys_write_register (Genesys_Device * dev, u_int8_t reg,
-			      u_int8_t val);
+sanei_genesys_write_register (Genesys_Device * dev, uint8_t reg,
+			      uint8_t val);
 
 extern SANE_Status
-sanei_genesys_get_status (Genesys_Device * dev, u_int8_t * status);
+sanei_genesys_get_status (Genesys_Device * dev, uint8_t * status);
 
 extern void sanei_genesys_init_fe (Genesys_Device * dev);
 
@@ -514,32 +655,38 @@ extern void sanei_genesys_init_structs (Genesys_Device * dev);
 extern SANE_Status
 sanei_genesys_init_shading_data (Genesys_Device * dev, int pixels_per_line);
 
+extern SANE_Status sanei_genesys_read_valid_words (Genesys_Device * dev,
+						  unsigned int *steps);
+
+extern SANE_Status sanei_genesys_read_scancnt (Genesys_Device * dev,
+						  unsigned int *steps);
+
 extern SANE_Status sanei_genesys_read_feed_steps (Genesys_Device * dev,
 						  unsigned int *steps);
 
 extern void
 sanei_genesys_calculate_zmode2 (SANE_Bool two_table,
-				u_int32_t exposure_time,
-				u_int16_t * slope_table,
+				uint32_t exposure_time,
+				uint16_t * slope_table,
 				int reg21,
-				int move, int reg22, u_int32_t * z1,
-				u_int32_t * z2);
+				int move, int reg22, uint32_t * z1,
+				uint32_t * z2);
 
 extern void
 sanei_genesys_calculate_zmode (Genesys_Device * dev,
-			       u_int32_t exposure_time,
-			       u_int32_t steps_sum,
-			       u_int16_t last_speed, u_int32_t feedl,
-			       u_int8_t fastfed, u_int8_t scanfed,
-			       u_int8_t fwdstep, u_int8_t tgtime,
-			       u_int32_t * z1, u_int32_t * z2);
+			       uint32_t exposure_time,
+			       uint32_t steps_sum,
+			       uint16_t last_speed, uint32_t feedl,
+			       uint8_t fastfed, uint8_t scanfed,
+			       uint8_t fwdstep, uint8_t tgtime,
+			       uint32_t * z1, uint32_t * z2);
 
 extern SANE_Status
-sanei_genesys_set_buffer_address (Genesys_Device * dev, u_int32_t addr);
+sanei_genesys_set_buffer_address (Genesys_Device * dev, uint32_t addr);
 
 extern SANE_Status
-sanei_genesys_fe_write_data (Genesys_Device * dev, u_int8_t addr,
-			     u_int16_t data);
+sanei_genesys_fe_write_data (Genesys_Device * dev, uint8_t addr,
+			     uint16_t data);
 
 extern SANE_Int
 sanei_genesys_exposure_time2 (Genesys_Device * dev,
@@ -549,17 +696,23 @@ sanei_genesys_exposure_time2 (Genesys_Device * dev,
 extern SANE_Int
 sanei_genesys_exposure_time (Genesys_Device * dev, Genesys_Register_Set * reg,
 			     int xdpi);
+extern SANE_Int
+sanei_genesys_generate_slope_table (uint16_t * slope_table, unsigned int max_steps,
+			      unsigned int use_steps, uint16_t stop_at,
+			      uint16_t vstart, uint16_t vend,
+			      unsigned int steps, double g,
+			      unsigned int *used_steps, unsigned int *vfinal);
 
 extern SANE_Int
 sanei_genesys_create_slope_table (Genesys_Device * dev,
-				  u_int16_t * slope_table, int steps,
+				  uint16_t * slope_table, int steps,
 				  int step_type, int exposure_time,
 				  SANE_Bool same_speed, double yres,
 				  int power_mode);
 
 SANE_Int
 sanei_genesys_create_slope_table3 (Genesys_Device * dev,
-				   u_int16_t * slope_table, int max_step,
+				   uint16_t * slope_table, int max_step,
 				   unsigned int use_steps,
 				   int step_type, int exposure_time,
 				   double yres,
@@ -568,7 +721,7 @@ sanei_genesys_create_slope_table3 (Genesys_Device * dev,
 				   int power_mode);
 
 extern void
-sanei_genesys_create_gamma_table (u_int16_t * gamma_table, int size,
+sanei_genesys_create_gamma_table (uint16_t * gamma_table, int size,
 				  float maximum, float gamma_max,
 				  float gamma);
 
@@ -577,19 +730,19 @@ extern SANE_Status sanei_genesys_start_motor (Genesys_Device * dev);
 extern SANE_Status sanei_genesys_stop_motor (Genesys_Device * dev);
 
 extern SANE_Status
-sanei_genesys_search_reference_point (Genesys_Device * dev, u_int8_t * data,
+sanei_genesys_search_reference_point (Genesys_Device * dev, uint8_t * data,
 				      int start_pixel, int dpi, int width,
 				      int height);
 
 extern SANE_Status
-sanei_genesys_write_pnm_file (char *filename, u_int8_t * data, int depth,
+sanei_genesys_write_pnm_file (char *filename, uint8_t * data, int depth,
 			      int channels, int pixels_per_line, int lines);
 
 extern SANE_Status
 sanei_genesys_test_buffer_empty (Genesys_Device * dev, SANE_Bool * empty);
 
 extern SANE_Status
-sanei_genesys_read_data_from_scanner (Genesys_Device * dev, u_int8_t * data,
+sanei_genesys_read_data_from_scanner (Genesys_Device * dev, uint8_t * data,
 				      size_t size);
 
 extern SANE_Status
