@@ -77,7 +77,7 @@
 
  ***************************************************************************/
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -85,15 +85,15 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <limits.h>
-#include "sane/sanei_jpeg.h"
+#include "../include/sane/sanei_jpeg.h"
 #include <sys/ioctl.h>
 
-#include "sane/sane.h"
-#include "sane/sanei.h"
-#include "sane/saneopts.h"
+#include "../include/sane/sane.h"
+#include "../include/sane/sanei.h"
+#include "../include/sane/saneopts.h"
 
 #define BACKEND_NAME	gphoto2
-#include "sane/sanei_backend.h"
+#include "../include/sane/sanei_backend.h"
 
 /* PSF 1/12/02 - gphoto2.h does a #include of config.h.  We don't have
  * config.h by that name (we call it sane/config.h), so the #undef of 
@@ -663,7 +663,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
   DBG_INIT ();
 
   DBG (1,
-       "GPHOTO2 Backend $Id: gphoto2.c,v 1.32 2005-12-03 18:23:55 psfales-guest Exp $\n");
+       "GPHOTO2 Backend $Id$\n");
 
   if (getenv ("GP_DEBUG"))
     {
@@ -671,7 +671,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
     }
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
 
   fp = sanei_config_open (GPHOTO2_CONFIG_FILE);
 
@@ -728,13 +728,21 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
 		}
 	      for (n = 0; n < entries; n++)
 		{
+#ifdef HAVE_GP_PORT_INFO_GET_PATH
+		  char *info_path = NULL;
+#endif
 		  result = gp_port_info_list_get_info (list, n, &info);
 		  if (result < 0)
 		    {
 		      gp_port_info_list_free (list);
 		      return SANE_STATUS_INVAL;
 		    }
+#ifdef HAVE_GP_PORT_INFO_GET_PATH
+		  gp_port_info_get_path (info, &info_path);
+		  if (strcmp (Cam_data.port, info_path) == 0)
+#else
 		  if (strcmp (Cam_data.port, info.path) == 0)
+#endif
 		    {
 		      break;
 		    }
