@@ -44,11 +44,17 @@
 */
 
 static char *hp_backend_version = "1.06";
-static char *hp_backend_revision = "$Revision: 1.21 $";
+static char *hp_backend_revision = "$Revision$";
 /* Changes:
 
    V 1.06:
-   $Log: hp.c,v $
+   $Log$
+   Revision 1.22  2008/11/26 21:21:25  kitno-guest
+   * backend/*.[ch]: nearly every backend used V_MAJOR
+   instead of SANE_CURRENT_MAJOR in sane_init()
+   * backend/snapscan.c: remove EXPECTED_VERSION check
+   since new SANE standard is forward compatible
+
    Revision 1.21  2004-10-04 18:09:05  kig-guest
    Rename global function hp_init_openfd to sanei_hp_init_openfd
 
@@ -226,17 +232,17 @@ static char *hp_backend_revision = "$Revision: 1.21 $";
 
 #define VERSIO                                8
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 #include "hp.h"
 
 #include <string.h>
 /* #include <sys/types.h> */
-/* #include <sane/sane.h> */
-#include "sane/sanei_config.h"
-#include "sane/sanei_backend.h"
-#include "sane/sanei_usb.h"
-#include "sane/sanei_thread.h"
-/* #include <sane/sanei_debug.h> */
+/* #include "../include/sane/sane.h" */
+#include "../include/sane/sanei_config.h"
+#include "../include/sane/sanei_backend.h"
+#include "../include/sane/sanei_usb.h"
+#include "../include/sane/sanei_thread.h"
+/* #include "../include/sane/sanei_debug.h" */
 #include "hp-device.h"
 #include "hp-handle.h"
 
@@ -816,7 +822,7 @@ sane_init (SANE_Int *version_code, SANE_Auth_Callback UNUSEDARG authorize)
   hp_destroy();
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, VERSIO);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, VERSIO);
 
   status = hp_init();
   DBG(3, "sane_init will finish with %s\n", sane_strstatus (status));
