@@ -76,7 +76,6 @@
 #include "ibm.h"
 #include "ibm-scsi.c"
 
-#define MM_PER_INCH	25.4
 #define MAX(a,b)	((a) > (b) ? (a) : (b))
 
 static int num_devices = 0;
@@ -544,11 +543,11 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 
 #if defined PACKAGE && defined VERSION
   DBG (2, "sane_init: ibm backend version %d.%d-%d ("
-       PACKAGE " " VERSION ")\n", V_MAJOR, V_MINOR, BUILD);
+       PACKAGE " " VERSION ")\n", SANE_CURRENT_MAJOR, V_MINOR, BUILD);
 #endif
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
 
   fp = sanei_config_open(IBM_CONFIG_FILE);
   if (fp)
@@ -900,8 +899,8 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
         }
 
       mode = s->val[OPT_MODE].s;
-      if ((strcmp (mode, "Lineart") == 0) ||
-	  (strcmp (mode, "Halftone")) == 0)
+      if ((strcmp (mode, SANE_VALUE_SCAN_MODE_LINEART) == 0) ||
+	  (strcmp (mode, SANE_VALUE_SCAN_MODE_HALFTONE)) == 0)
         {
           s->params.format = SANE_FRAME_GRAY;
           s->params.bytes_per_line = s->params.pixels_per_line / 8;
@@ -909,7 +908,7 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
           s->params.pixels_per_line = s->params.bytes_per_line * 8;
           s->params.depth = 1;
         }
-      else /* if (strcmp (mode, "Gray") == 0) */
+      else /* if (strcmp (mode, SANE_VALUE_SCAN_MODE_GRAY) == 0) */
         {
           s->params.format = SANE_FRAME_GRAY;
           s->params.bytes_per_line = s->params.pixels_per_line;
@@ -967,15 +966,15 @@ sane_start (SANE_Handle handle)
   s->brightness = s->val[OPT_BRIGHTNESS].w;
   s->contrast = s->val[OPT_CONTRAST].w;
   s->bpp = s->params.depth;
-  if (strcmp (mode_str, "Lineart") == 0) 
+  if (strcmp (mode_str, SANE_VALUE_SCAN_MODE_LINEART) == 0) 
     {
       s->image_composition = IBM_BINARY_MONOCHROME;
     }
-  else if (strcmp (mode_str, "Halftone") == 0)
+  else if (strcmp (mode_str, SANE_VALUE_SCAN_MODE_HALFTONE) == 0)
     {
       s->image_composition = IBM_DITHERED_MONOCHROME;
     }
-  else if (strcmp (mode_str, "Gray") == 0)
+  else if (strcmp (mode_str, SANE_VALUE_SCAN_MODE_GRAY) == 0)
     {
       s->image_composition = IBM_GRAYSCALE;
     }
