@@ -2,7 +2,7 @@
 
    (C) 2003-2004 Lexmark International, Inc. (Original Source code)
    (C) 2005 Fred Odendaal
-   (C) 2006-2007 Stphane Voltz <stef.dev@free.fr>
+   (C) 2006-2009 Stphane Voltz <stef.dev@free.fr>
    
    This file is part of the SANE package.
 
@@ -44,34 +44,10 @@
 
    **************************************************************************/
 
-#include "../include/sane/config.h"
-
-#include <errno.h>
-#include <signal.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <sys/types.h>
-#include <sys/wait.h>
-#include <time.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <sys/time.h>
-
-
-#include "../include/sane/sane.h"
-#include "../include/sane/sanei.h"
-#include "../include/sane/saneopts.h"
-#include "../include/sane/sanei_config.h"
-#include "../include/sane/sanei_usb.h"
-
 #include "lexmark.h"
-#include "../include/sane/sanei_backend.h"
-
 
 #define LEXMARK_CONFIG_FILE "lexmark.conf"
-#define BUILD 19
+#define BUILD 21
 #define MAX_OPTION_STRING_SIZE 255
 
 static Lexmark_Device *first_lexmark_device = 0;
@@ -188,10 +164,10 @@ init_options (Lexmark_Device * dev)
       od->constraint.word_list = a920_dpi_list;
       break;
     case X1100_B2_SENSOR:
-      od->constraint.word_list = x1100_dpi_list;
       od->constraint.word_list = x1100_dpi_list;
       break;
     case X1200_SENSOR:
+    case X1200_USB2_SENSOR:
       od->constraint.word_list = x1200_dpi_list;
       break;
     }
@@ -359,7 +335,7 @@ SANE_Status
 attachLexmark (SANE_String_Const devname)
 {
   Lexmark_Device *lexmark_device;
-  SANE_Int dn, vendor, product;
+  SANE_Int dn, vendor, product, variant;
   SANE_Status status;
 
   DBG (2, "attachLexmark: devname=%s\n", devname);
@@ -393,8 +369,10 @@ attachLexmark (SANE_String_Const devname)
   status = SANE_STATUS_GOOD;
   /* put the id of the model you want to fake here */
   vendor = 0x043d;
-  product = 0x007d;	/* X12xx */
+  product = 0x007c;	/* X11xx */
+  variant = 0xb2;
 #else
+  variant = 0;
   status = sanei_usb_get_vendor_product (dn, &vendor, &product);
 #endif
   if (status != SANE_STATUS_GOOD)
@@ -411,13 +389,13 @@ attachLexmark (SANE_String_Const devname)
   sanei_usb_close (dn);
 #endif
 
-  DBG (2, "attachLexmark: testing device `%s': 0x%04x:0x%04x, no variant\n",
-       devname, vendor, product);
+  DBG (2, "attachLexmark: testing device `%s': 0x%04x:0x%04x, variant=%d\n",
+       devname, vendor, product, variant);
   if (sanei_lexmark_low_assign_model (lexmark_device,
 			  		devname,
 					vendor,
 					product,
-					0) != SANE_STATUS_GOOD)
+					variant) != SANE_STATUS_GOOD)
     {
       DBG (2, "attachLexmark: unsupported device `%s': 0x%04x:0x%04x\n",
 	   devname, vendor, product);
@@ -466,14 +444,14 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 
   DBG_INIT ();
 
-  DBG (1, "SANE Lexmark backend version %d.%d-rc2-%d\n", V_MAJOR, V_MINOR, BUILD);
+  DBG (1, "SANE Lexmark backend version %d.%d.%d-devel\n", SANE_CURRENT_MAJOR, V_MINOR, BUILD);
 
   auth_callback = authorize;
 
   DBG (2, "sane_init: version_code=%p\n", (void *) version_code);
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, BUILD);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, BUILD);
 
 #ifndef FAKE_USB
   sanei_usb_init ();
@@ -1082,6 +1060,7 @@ sane_start (SANE_Handle handle)
   Lexmark_Device *lexmark_device;
   SANE_Int offset;
   SANE_Status status;
+  int resolution;
 
   DBG (2, "sane_start: handle=%p\n", (void *) handle);
 
@@ -1133,8 +1112,13 @@ sane_start (SANE_Handle handle)
       sanei_lexmark_low_search_home_bwd (lexmark_device);
     }
 
-  /* do calibration before offset detection */
-  sanei_lexmark_low_set_scan_regs (lexmark_device, 0, SANE_FALSE);
+  /* do calibration before offset detection , use sensor max dpi, not motor's one */
+  resolution = lexmark_device->val[OPT_RESOLUTION].w;
+  if(resolution > 600)
+  {
+	  resolution = 600;
+  }
+  sanei_lexmark_low_set_scan_regs (lexmark_device, resolution, 0, SANE_FALSE);
   status = sanei_lexmark_low_calibration (lexmark_device);
   if (status != SANE_STATUS_GOOD)
     {
@@ -1154,7 +1138,7 @@ sane_start (SANE_Handle handle)
   /* Set the shadow registers for scan with the options (resolution, mode, 
      size) set in the front end. Pass the offset so we can get the vert.
      start. */
-  sanei_lexmark_low_set_scan_regs (lexmark_device, offset, SANE_TRUE);
+  sanei_lexmark_low_set_scan_regs (lexmark_device, lexmark_device->val[OPT_RESOLUTION].w, offset, SANE_TRUE);
   
   if (sanei_lexmark_low_start_scan (lexmark_device) == SANE_STATUS_GOOD)
     {
