@@ -15,7 +15,7 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-  $Id: niash.c,v 1.13 2006-02-04 10:28:50 ullsig-guest Exp $
+  $Id$
 */
 
 /*
@@ -200,9 +200,9 @@ static const SANE_Range rangeXmm = { 0, 220, 1 };
 static const SANE_Range rangeYmm = { 0, 297, 1 };
 static const SANE_Int startUpGamma = SANE_FIX (1.6);
 
-static const char colorStr[] = { "Color" };
-static const char grayStr[] = { "Gray" };
-static const char lineartStr[] = { "Lineart" };
+static const char colorStr[] = { SANE_VALUE_SCAN_MODE_COLOR };
+static const char grayStr[] = { SANE_VALUE_SCAN_MODE_GRAY };
+static const char lineartStr[] = { SANE_VALUE_SCAN_MODE_LINEART };
 
 #define DEPTH_LINEART  1
 #define DEPTH_GRAY     8
@@ -834,7 +834,7 @@ sane_init (SANE_Int * piVersion, SANE_Auth_Callback pfnAuth)
 
   if (piVersion != NULL)
     {
-      *piVersion = SANE_VERSION_CODE (V_MAJOR, V_MINOR, BUILD);
+      *piVersion = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, BUILD);
     }
 
   /* initialise transfer methods */
