@@ -1,5 +1,6 @@
 /* SANE - Scanner Access Now Easy.
 
+   Copyright (C) 2007-2008 Nicolas Martin, <nicols-guest at alioth dot debian dot org>
    Copyright (C) 2006-2007 Wittawat Yamwong <wittawat@web.de>
 
    This file is part of the SANE package.
@@ -40,7 +41,7 @@
    whether to permit this exception to apply to your modifications.
    If you do not wish that, delete this exception notice.
  */
-# include "../include/sane/config.h"
+#include "../include/sane/config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -65,11 +66,13 @@
 extern const pixma_config_t pixma_mp150_devices[];
 extern const pixma_config_t pixma_mp750_devices[];
 extern const pixma_config_t pixma_mp730_devices[];
+extern const pixma_config_t pixma_iclass_devices[];
 
 static const pixma_config_t *const pixma_devices[] = {
   pixma_mp150_devices,
   pixma_mp750_devices,
   pixma_mp730_devices,
+  pixma_iclass_devices,
   NULL
 };
 
@@ -86,7 +89,8 @@ u8tohex (uint8_t x, char *str)
 {
   static const char hdigit[16] =
     { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd',
-'e', 'f' };
+    'e', 'f'
+    };
   str[0] = hdigit[(x >> 4) & 0xf];
   str[1] = hdigit[x & 0xf];
   str[2] = '\0';
@@ -105,34 +109,41 @@ void
 pixma_hexdump (int level, const void *d_, unsigned len)
 {
   const uint8_t *d = (const uint8_t *) (d_);
-  unsigned ofs, c;
+  unsigned ofs, c, plen;
   char line[100];		/* actually only 1+8+1+8*3+1+8*3+1 = 61 bytes needed */
 
   if (level > debug_level)
     return;
+  if (level == debug_level)
+    /* if debuglevel == exact match and buffer contains more than 3 lines, print 2 lines + .... */
+    plen = (len > 64) ? 32: len;
+  else
+    plen = len;
   ofs = 0;
-  while (ofs < len)
+  while (ofs < plen)
     {
       char *p;
       line[0] = ' ';
       u32tohex (ofs, line + 1);
       line[9] = ':';
       p = line + 10;
-      for (c = 0; c != 16 && (ofs + c) < len; c++)
-	{
-	  u8tohex (d[ofs + c], p);
-	  p[2] = ' ';
-	  p += 3;
-	  if (c == 7)
-	    {
-	      p[0] = ' ';
-	      p++;
-	    }
-	}
+      for (c = 0; c != 16 && (ofs + c) < plen; c++)
+        {
+          u8tohex (d[ofs + c], p);
+          p[2] = ' ';
+          p += 3;
+          if (c == 7)
+            {
+              p[0] = ' ';
+              p++;
+            }
+        }
       p[0] = '\0';
       pixma_dbg (level, "%s\n", line);
       ofs += c;
     }
+  if (len > plen)
+    pixma_dbg(level, "......\n");
 }
 
 static void
@@ -217,11 +228,13 @@ pixma_strerror (int error)
     case PIXMA_EPROTO:
       return "EPROTO";
     case PIXMA_EPAPER_JAMMED:
-      return "EPAPAR_JAMMED";
+      return "EPAPER_JAMMED";
     case PIXMA_ECOVER_OPEN:
       return "ECOVER_OPEN";
     case PIXMA_ENO_PAPER:
       return "ENO_PAPER";
+    case PIXMA_EOF:
+      return "EEOF";
     }
   snprintf (buf, sizeof (buf), "EUNKNOWN:%d", error);
   return buf;
@@ -322,21 +335,21 @@ pixma_check_result (pixma_cmdbuf_t * cb)
     {
       error = pixma_map_status_errno (pixma_get_be16 (r));
       if (expected_reslen != 0)
-	{
-	  if (len == expected_reslen)
-	    {
-	      if (pixma_sum_bytes (r + header_len, len - header_len) != 0)
-		error = PIXMA_EPROTO;
-	    }
-	  else
-	    {
-	      /* This case will happen when a command cannot be completely
-	         executed, e.g. because you press the cancel button. The
-	         device will return only a header with PIXMA_STATUS_FAILED. */
-	      if (len != header_len)
-		error = PIXMA_EPROTO;
-	    }
-	}
+        {
+          if (len == expected_reslen)
+            {
+              if (pixma_sum_bytes (r + header_len, len - header_len) != 0)
+                 error = PIXMA_EPROTO;
+            }
+          else
+            {
+              /* This case will happen when a command cannot be completely
+                 executed, e.g. because you press the cancel button. The
+                 device will return only a header with PIXMA_STATUS_FAILED. */
+              if (len != header_len)
+                 error = PIXMA_EPROTO;
+            }
+        }
     }
   else
     error = PIXMA_EPROTO;
@@ -362,13 +375,13 @@ pixma_cmd_transaction (pixma_t * s, const void *cmd, unsigned cmdlen,
   if (error != (int) cmdlen)
     {
       if (error >= 0)
-	{
-	  /* Write timeout is too low? */
-	  PDBG (pixma_dbg
-		(1, "ERROR:incomplete write, %u out of %u written\n",
-		 (unsigned) error, cmdlen));
-	  error = PIXMA_ETIMEDOUT;
-	}
+        {
+          /* Write timeout is too low? */
+          PDBG (pixma_dbg
+          (1, "ERROR:incomplete write, %u out of %u written\n",
+           (unsigned) error, cmdlen));
+          error = PIXMA_ETIMEDOUT;
+        }
       return error;
     }
 
@@ -385,7 +398,7 @@ pixma_cmd_transaction (pixma_t * s, const void *cmd, unsigned cmdlen,
     {
       error = pixma_read (s->io, data, expected_len);
       if (error == PIXMA_ETIMEDOUT)
-	PDBG (pixma_dbg (2, "No response yet. Timed out in %d sec.\n", tmo));
+        PDBG (pixma_dbg (2, "No response yet. Timed out in %d sec.\n", tmo));
     }
   while (error == PIXMA_ETIMEDOUT && --tmo != 0);
   if (error < 0)
@@ -393,10 +406,8 @@ pixma_cmd_transaction (pixma_t * s, const void *cmd, unsigned cmdlen,
       PDBG (pixma_dbg (1, "WARNING:Error in response phase. cmd:%02x%02x\n",
 		       ((const uint8_t *) cmd)[0],
 		       ((const uint8_t *) cmd)[1]));
-      PDBG (pixma_dbg
-	    (1,
-	     "   If the scanner hangs, reset it and/or unplug the "
-	     "USB cable.\n"));
+      PDBG (pixma_dbg (1,"  If the scanner hangs, reset it and/or unplug the "
+	                       "USB cable.\n"));
     }
   return error;			/* length of the result packet or error */
 }
@@ -501,6 +512,9 @@ pixma_open (unsigned devnr, pixma_t ** handle)
   s->ops = s->cfg->ops;
   s->scanning = 0;
   error = s->ops->open (s);
+  if (error < 0)
+    goto rollback;
+  error = pixma_deactivate (s->io);
   if (error < 0)
     goto rollback;
   *handle = s;
@@ -618,67 +632,65 @@ pixma_read_image (pixma_t * s, void *buf, unsigned len)
   if (s->underrun)
     {
       if (s->cur_image_size < s->param->image_size)
-	{
-	  ib.wptr = fill_pixels (s, ib.wptr, ib.wend, 0xff);
-	}
+        {
+          ib.wptr = fill_pixels (s, ib.wptr, ib.wend, 0xff);
+        }
       else
-	{
-	  PDBG (pixma_dbg
-		(3, "pixma_read_image():completed (underrun detected)\n"));
-	  s->scanning = 0;
-	}
+        {
+          PDBG (pixma_dbg
+          (3, "pixma_read_image():completed (underrun detected)\n"));
+          s->scanning = 0;
+        }
       return ib.wptr - (uint8_t *) buf;
     }
 
   while (ib.wptr != ib.wend)
     {
       if (ib.rptr == ib.rend)
-	{
-	  ib.rptr = ib.rend = NULL;
-	  result = s->ops->fill_buffer (s, &ib);
-	  if (result < 0)
-	    goto cancel;
-	  if (result == 0)
-	    {			/* end of image? */
-	      s->ops->finish_scan (s);
-#ifndef NDEBUG
-	      if (s->cur_image_size != s->param->image_size)
-		{
-		  pixma_dbg (1, "WARNING:image size mismatches\n");
-		  pixma_dbg (1,
-			     "    %u expected (%d lines) but %u received (%d lines)\n",
-			     s->param->image_size, s->param->h,
-			     s->cur_image_size,
-			     s->cur_image_size / s->param->line_size);
-		  if ((s->cur_image_size % s->param->line_size) != 0)
-		    {
-		      pixma_dbg (1,
-				 "BUG:received data not multiple of line_size\n");
-		    }
-		}
-#endif /* !NDEBUG */
-	      if (s->cur_image_size < s->param->image_size)
-		{
-		  s->underrun = 1;
-		  ib.wptr = fill_pixels (s, ib.wptr, ib.wend, 0xff);
-		}
-	      else
-		{
-		  PDBG (pixma_dbg (3, "pixma_read_image():completed\n"));
-		  s->scanning = 0;
-		}
-	      break;
-	    }
-	  s->cur_image_size += result;
-	  PASSERT (s->cur_image_size <= s->param->image_size);
-	}
+        {
+          ib.rptr = ib.rend = NULL;
+          result = s->ops->fill_buffer (s, &ib);
+          if (result < 0)
+            goto cancel;
+          if (result == 0)
+            {			/* end of image? */
+              s->ops->finish_scan (s);
+              if (s->cur_image_size != s->param->image_size)
+                {
+                  pixma_dbg (1, "WARNING:image size mismatches\n");
+                  pixma_dbg (1,
+                       "    %u expected (%d lines) but %u received (%d lines)\n",
+                       s->param->image_size, s->param->h,
+                       s->cur_image_size,
+                       s->cur_image_size / s->param->line_size);
+                  if ((s->cur_image_size % s->param->line_size) != 0)
+                    {
+                      pixma_dbg (1,
+                     "BUG:received data not multiple of line_size\n");
+                    }
+                }
+              if (s->cur_image_size < s->param->image_size)
+                {
+                  s->underrun = 1;
+                  ib.wptr = fill_pixels (s, ib.wptr, ib.wend, 0xff);
+                }
+              else
+                {
+                  PDBG (pixma_dbg (3, "pixma_read_image():completed\n"));
+                  s->scanning = 0;
+                }
+              break;
+            }
+          s->cur_image_size += result;
+          PASSERT (s->cur_image_size <= s->param->image_size);
+        }
       if (ib.rptr)
-	{
-	  unsigned count = MIN (ib.rend - ib.rptr, ib.wend - ib.wptr);
-	  memcpy (ib.wptr, ib.rptr, count);
-	  ib.rptr += count;
-	  ib.wptr += count;
-	}
+        {
+          unsigned count = MIN (ib.rend - ib.rptr, ib.wend - ib.wptr);
+          memcpy (ib.wptr, ib.rptr, count);
+          ib.rptr += count;
+          ib.wptr += count;
+        }
     }
   s->imagebuf = ib;		/* store rptr and rend */
   return ib.wptr - (uint8_t *) buf;
@@ -711,7 +723,19 @@ pixma_enable_background (pixma_t * s, int enabled)
   return pixma_set_interrupt_mode (s->io, enabled);
 }
 
-unsigned
+int
+pixma_activate_connection(pixma_t * s)
+{
+  return pixma_activate (s->io);
+}
+
+int
+pixma_deactivate_connection(pixma_t * s)
+{
+  return pixma_deactivate (s->io);
+}
+
+uint32_t
 pixma_wait_event (pixma_t * s, int timeout /*ms */ )
 {
   unsigned events;
@@ -747,6 +771,9 @@ pixma_check_scan_param (pixma_t * s, pixma_scan_param_t * sp)
 	(sp->xdpi == s->cfg->xdpi && sp->ydpi == s->cfg->ydpi)))
     return PIXMA_EINVAL;
 
+  if (s->ops->check_param (s, sp) < 0)
+    return PIXMA_EINVAL;
+
   /* FIXME: I assume the same minimum width and height for every model. */
   CLAMP2 (sp->x, sp->w, 13, s->cfg->width, sp->xdpi);
   CLAMP2 (sp->y, sp->h, 8, s->cfg->height, sp->ydpi);
@@ -758,37 +785,37 @@ pixma_check_scan_param (pixma_t * s, pixma_scan_param_t * sp)
 
     case PIXMA_SOURCE_TPU:
       if ((s->cfg->cap & PIXMA_CAP_TPU) != PIXMA_CAP_TPU)
-	{
-	  sp->source = PIXMA_SOURCE_FLATBED;
-	  PDBG (pixma_dbg
-		(1, "WARNING: TPU unsupported, fallback to flatbed.\n"));
-	}
+        {
+          sp->source = PIXMA_SOURCE_FLATBED;
+          PDBG (pixma_dbg
+          (1, "WARNING: TPU unsupported, fallback to flatbed.\n"));
+        }
       break;
 
     case PIXMA_SOURCE_ADF:
       if ((s->cfg->cap & PIXMA_CAP_ADF) != PIXMA_CAP_ADF)
-	{
-	  sp->source = PIXMA_SOURCE_FLATBED;
-	  PDBG (pixma_dbg
-		(1, "WARNING: ADF unsupported, fallback to flatbed.\n"));
-	}
+        {
+          sp->source = PIXMA_SOURCE_FLATBED;
+          PDBG (pixma_dbg
+          (1, "WARNING: ADF unsupported, fallback to flatbed.\n"));
+        }
       break;
 
     case PIXMA_SOURCE_ADFDUP:
       if ((s->cfg->cap & PIXMA_CAP_ADFDUP) != PIXMA_CAP_ADFDUP)
-	{
-	  if (s->cfg->cap & PIXMA_CAP_ADF)
-	    {
-	      sp->source = PIXMA_SOURCE_ADF;
-	    }
-	  else
-	    {
-	      sp->source = PIXMA_SOURCE_FLATBED;
-	    }
-	  PDBG (pixma_dbg
-		(1, "WARNING: ADF duplex unsupported, fallback to %d.\n",
-		 sp->source));
-	}
+        {
+          if (s->cfg->cap & PIXMA_CAP_ADF)
+            {
+              sp->source = PIXMA_SOURCE_ADF;
+            }
+          else
+            {
+              sp->source = PIXMA_SOURCE_FLATBED;
+            }
+          PDBG (pixma_dbg
+          (1, "WARNING: ADF duplex unsupported, fallback to %d.\n",
+           sp->source));
+        }
       break;
     }
 
@@ -844,9 +871,9 @@ pixma_fill_gamma_table (double gamma, uint8_t * table, unsigned n)
 }
 
 int
-pixma_find_scanners (void)
+pixma_find_scanners (const char **conf_devices)
 {
-  return pixma_collect_devices (pixma_devices);
+  return pixma_collect_devices (conf_devices, pixma_devices);
 }
 
 const char *
